@@ -7,6 +7,7 @@ use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::SNMP;
 use FusionInventory::Agent::Tools::Network;
 use FusionInventory::Agent::SNMP::MibSupport;
+use FusionInventory::Agent::SNMP::Device::Components;
 
 # Supported infos are specified here:
 # http://fusioninventory.org/documentation/dev/spec/protocol/netdiscovery.html
@@ -16,7 +17,7 @@ use constant discovery => [ qw(
     )];
 # http://fusioninventory.org/documentation/dev/spec/protocol/netinventory.html
 use constant inventory => [ qw(
-        INFO PORTS MODEMS FIRMWARES SIMCARDS PAGECOUNTERS CARTRIDGES
+        INFO PORTS MODEMS FIRMWARES SIMCARDS PAGECOUNTERS CARTRIDGES COMPONENTS
     )];
 
 # common base variables
@@ -56,11 +57,11 @@ my $inventory_only_base_variables = {
         type => 'memory',
     },
     RAM          => {
-        oid  => [
-            '.1.3.6.1.4.1.2021.4.5',
-            '.1.3.6.1.4.1.9.3.6.6.0',
-            '.1.3.6.1.2.1.25.2.2.0',
-        ],
+        oid  => {
+            '.1.3.6.1.4.1.2021.4.5'     => 'kb',
+            '.1.3.6.1.4.1.9.3.6.6.0'    => 'bytes',
+            '.1.3.6.1.2.1.25.2.2.0'     => 'kb',
+        },
         type => 'memory',
     },
 };
@@ -106,7 +107,8 @@ sub loadMibSupport {
     # identify device supported MIBs
     $self->{MIBSUPPORT} = FusionInventory::Agent::SNMP::MibSupport->new(
         sysobjectid  => $sysobjectid,
-        device       => $self
+        device       => $self,
+        logger       => $self->{logger}
     );
 }
 
@@ -188,6 +190,14 @@ sub getInventory {
     return $inventory;
 }
 
+sub addComponent {
+    my ($self, $component) = @_;
+
+    return unless _cleanHash($component);
+
+    push @{$self->{COMPONENTS}->{COMPONENT}}, $component;
+}
+
 sub addModem {
     my ($self, $modem) = @_;
 
@@ -232,7 +242,7 @@ sub _cleanHash {
         $keys++;
         next if defined($hashref->{$key});
         delete $hashref->{$key};
-        $keys--,
+        $keys--;
     }
 
     return $keys;
@@ -262,6 +272,10 @@ sub setSerial {
             '.1.3.6.1.4.1.318.1.1.4.1.5.0',          # MasterSwitch-MIB
             '.1.3.6.1.4.1.6027.3.8.1.1.5.0',         # F10-C-SERIES-CHASSIS-MIB
             '.1.3.6.1.4.1.6027.3.10.1.2.2.1.12.1',   # FORCE10-SMI
+            '.1.3.6.1.4.1.3417.2.11.1.4.0',          # BLUECOAT-SG-PROXY-MIB
+            '.1.3.6.1.4.1.232.2.2.2.1.0',            # CPQSINFO-MIB
+            '.1.3.6.1.4.1.232.11.2.10.3.0',          # CPQHOST-MIB
+            '.1.3.6.1.4.1.14823.2.3.3.1.2.1.1.4.144.76.129.206.143.6', # ARUBA-MIB
         );
         foreach my $oid (@oids) {
             $serial = $self->get($oid);
@@ -440,7 +454,7 @@ sub setType {
 sub setManufacturer {
     my ($self) = @_;
 
-    # Permit mib support to reset type
+    # Permit mib support to reset manufacturer
     if ($self->{MIBSUPPORT}) {
         my $manufacturer = $self->{MIBSUPPORT}->getMethod('getManufacturer');
         $self->{MANUFACTURER} = $manufacturer if $manufacturer;
@@ -470,11 +484,26 @@ sub _set_from_oid_list {
 
     foreach my $key (keys %{$list}) {
         my $variable = $list->{$key};
+        my $type = $variable->{type};
 
         my $raw_value;
         if (ref $variable->{oid} eq 'ARRAY') {
             foreach my $oid (@{$variable->{oid}}) {
                 $raw_value = $self->get($oid);
+                if (defined($raw_value) && $type =~ /^memory|count$/) {
+                    # Skip value if it seems not to include a number for memory & count
+                    undef $raw_value unless $raw_value =~ /\d+/;
+                }
+                last if defined $raw_value;
+            }
+        } elsif (ref $variable->{oid} eq 'HASH') {
+            foreach my $oid (keys %{$variable->{oid}}) {
+                $raw_value = $self->get($oid);
+                if (defined($raw_value) && $type =~ /^memory|count$/) {
+                    # Skip value if it seems not to include a number for memory & count
+                    undef $raw_value unless $raw_value =~ /\d+/;
+                    $raw_value .= ' kB' if $variable->{oid}->{$oid} eq 'kb' && isInteger($raw_value);
+                }
                 last if defined $raw_value;
             }
         } else {
@@ -482,7 +511,6 @@ sub _set_from_oid_list {
         }
         next unless defined $raw_value;
 
-        my $type = $variable->{type};
         my $value =
             $type eq 'memory' ? getCanonicalMemory($raw_value) :
             $type eq 'string' ? getCanonicalString($raw_value) :
@@ -521,6 +549,28 @@ sub setIp {
     $self->{IPS}->{IP} = [ $ip ] if $ip;
 }
 
+sub setComponents {
+    my ($self) = @_;
+
+    # First try to get components from standard ENTITY-MIB support
+    my $components = FusionInventory::Agent::SNMP::Device::Components->new(
+        device => $self
+    );
+    if ($components) {
+        foreach my $component (@{$components->getPhysicalComponents()}) {
+            $self->addComponent($component);
+        }
+    }
+
+    # Then look for any plugin providing getComponents API
+    if ($self->{MIBSUPPORT}) {
+        $components = $self->{MIBSUPPORT}->getMethod('getComponents') || [];
+        foreach my $component (@{$components}) {
+            $self->addComponent($component);
+        }
+    }
+}
+
 1;
 
 __END__
