@@ -3,6 +3,8 @@ package FusionInventory::Agent::Task::Inventory::MacOS::Storages;
 use strict;
 use warnings;
 
+use parent 'FusionInventory::Agent::Task::Inventory::Module';
+
 use Scalar::Util qw/looks_like_number/;
 
 use FusionInventory::Agent::Tools;
@@ -99,13 +101,7 @@ sub _getSerialATAStorages {
         my $storage = _extractStorage($hash);
         $storage->{TYPE} = 'Disk drive';
         $storage->{INTERFACE} = 'SERIAL-ATA';
-        %$storage = map {
-            my $value = $storage->{$_};
-            $value =~ s/^\s*//g;
-            $value =~ s/\s*$//g;
-            $_ => $value
-        } keys %$storage;
-        push @storages, $storage;
+        push @storages, _sanitizedHash($storage);
     }
 
     return @storages;
@@ -147,13 +143,7 @@ sub _getDiscBurningStorages {
     for my $hash (values %{$infos->{storages}}) {
         my $storage = _extractDiscBurning($hash);
         $storage->{TYPE} = 'Disk burning';
-        %$storage = map {
-            my $value = $storage->{$_};
-            $value =~ s/^\s*//g;
-            $value =~ s/\s*$//g;
-            $_ => $value
-        } keys %$storage;
-        push @storages, $storage;
+        push @storages, _sanitizedHash($storage);
     }
 
     return @storages;
@@ -198,13 +188,7 @@ sub _getCardReaderStorages {
             $storage = _extractSdCard($hash);
             $storage->{TYPE} = 'SD Card';
         }
-        %$storage = map {
-            my $value = $storage->{$_};
-            $value =~ s/^\s*//g;
-            $value =~ s/\s*$//g;
-            $_ => $value
-        } keys %$storage;
-        push @storages, $storage;
+        push @storages, _sanitizedHash($storage);
     }
 
     return @storages;
@@ -304,13 +288,7 @@ sub _getUSBStorages {
         my $storage = _extractUSBStorage($hash);
         $storage->{TYPE} = 'Disk drive';
         $storage->{INTERFACE} = 'USB';
-        %$storage = map {
-            my $value = $storage->{$_} || '';
-            $value =~ s/^\s*//g;
-            $value =~ s/\s*$//g;
-            $_ => $value
-        } keys %$storage;
-        push @storages, $storage;
+        push @storages, _sanitizedHash($storage);
     }
 
     return @storages;
@@ -340,24 +318,6 @@ sub _extractDiskSize {
             getCanonicalSize($hash->{size}, 1024);
 }
 
-sub _cleanSizeString {
-    my ($sizeString) = @_;
-
-    return unless $sizeString =~ /^(\d+(?:(?:\.|,)\d+)?) /;
-    my $nbStr = $1;
-    $nbStr =~ s/,/./;
-
-    return $nbStr;
-}
-
-sub _getSizeUnit {
-    my ($sizeString) = @_;
-
-    return 'GB' if $sizeString =~ /GB$/;
-    return 'MB' if $sizeString =~ /MB$/;
-    return '';
-}
-
 sub _extractValueInHashWithKeyPattern {
     my ($pattern, $hash) = @_;
 
@@ -385,13 +345,7 @@ sub _getFireWireStorages {
         my $storage = _extractFireWireStorage($hash);
         $storage->{TYPE} = 'Disk drive';
         $storage->{INTERFACE} = 'FireWire';
-        %$storage = map {
-            my $value = $storage->{$_} || '';
-            $value =~ s/^\s*//g;
-            $value =~ s/\s*$//g;
-            $_ => $value
-        } keys %$storage;
-        push @storages, $storage;
+        push @storages, _sanitizedHash($storage);
     }
 
     return @storages;
@@ -413,4 +367,16 @@ sub _extractFireWireStorage {
     return $storage;
 }
 
+sub _sanitizedHash {
+    my ($hash) = @_;
+    foreach my $key (keys(%{$hash})) {
+        if (defined($hash->{$key})) {
+            $hash->{$key} = trimWhitespace($hash->{$key});
+        } else {
+            delete $hash->{$key};
+        }
+    }
+    return $hash;
+}
+
 1;
